@@ -33,6 +33,8 @@ import java.nio.charset.StandardCharsets;
 import java.nio.charset.UnmappableCharacterException;
 import java.nio.charset.UnsupportedCharsetException;
 
+import org.eclipse.osgi.util.NLS;
+
 import org.eclipse.core.filesystem.EFS;
 import org.eclipse.core.filesystem.IFileInfo;
 import org.eclipse.core.filesystem.IFileStore;
@@ -353,9 +355,7 @@ public class FileStoreTextFileBuffer extends FileStoreFileBuffer implements ITex
 				if (fEncoding == null)
 					fEncoding= description.getCharset();
 			}
-		} catch (CoreException e) {
-			// do nothing
-		} catch (IOException e) {
+		} catch (CoreException | IOException e) {
 			// do nothing
 		}
 
@@ -534,9 +534,15 @@ public class FileStoreTextFileBuffer extends FileStoreFileBuffer implements ITex
 			StringBuilder buffer= new StringBuilder(BUFFER_SIZE);
 			char[] readBuffer= new char[READER_CHUNK_SIZE];
 			int n= in.read(readBuffer);
-			while (n > 0) {
-				buffer.append(readBuffer, 0, n);
-				n= in.read(readBuffer);
+			try {
+				while (n > 0) {
+					buffer.append(readBuffer, 0, n);
+					n= in.read(readBuffer);
+				}
+			} catch (OutOfMemoryError e) {
+				// give the JVM a hint that it can free the big buffer right away
+				buffer= null;
+				throw new IOException(NLS.bind(FileBuffersMessages.ResourceTextFileBuffer_oom_on_file_read, file.toURI()), e);
 			}
 
 			document.set(buffer.toString());
