@@ -27,9 +27,12 @@ import java.nio.charset.Charset;
 import java.nio.charset.CharsetEncoder;
 import java.nio.charset.CodingErrorAction;
 import java.nio.charset.IllegalCharsetNameException;
+import java.nio.charset.StandardCharsets;
 import java.nio.charset.UnmappableCharacterException;
 import java.nio.charset.UnsupportedCharsetException;
 
+import org.eclipse.osgi.util.NLS;
+
 import org.eclipse.core.runtime.Assert;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IProgressMonitor;
@@ -97,20 +100,6 @@ public class ResourceTextFileBuffer extends ResourceFileBuffer implements ITextF
 	 * Constant for representing the error status. This is considered a value object.
 	 */
 	static final private IStatus STATUS_ERROR= new Status(IStatus.ERROR, FileBuffersPlugin.PLUGIN_ID, IStatus.OK, FileBuffersMessages.FileBuffer_status_error, null);
-	/**
-	 * Constant denoting UTF-8 encoding.
-	 */
-	private static final String CHARSET_UTF_8= "UTF-8"; //$NON-NLS-1$
-	/**
-	 * Constant denoting UTF-16 encoding.
-	 * @since 3.4
-	 */
-	private static final String CHARSET_UTF_16= "UTF-16"; //$NON-NLS-1$
-	/**
-	 * Constant denoting UTF-16LE encoding.
-	 * @since 3.4
-	 */
-	private static final String CHARSET_UTF_16LE= "UTF-16LE"; //$NON-NLS-1$
 
 	/**
 	 * Constant denoting an empty set of properties
@@ -311,8 +300,8 @@ public class ResourceTextFileBuffer extends ResourceFileBuffer implements ITextF
 
 		String encoding= computeEncoding();
 
-		if (fBOM == IContentDescription.BOM_UTF_16LE && CHARSET_UTF_16.equals(encoding))
-			encoding= CHARSET_UTF_16LE;
+		if (fBOM == IContentDescription.BOM_UTF_16LE && StandardCharsets.UTF_16.name().equals(encoding))
+			encoding= StandardCharsets.UTF_16LE.name();
 
 		Charset charset;
 		try {
@@ -345,7 +334,7 @@ public class ResourceTextFileBuffer extends ResourceFileBuffer implements ITextF
 			stream= new ByteArrayInputStream(bytes, 0, byteBuffer.limit());
 		} catch (CharacterCodingException ex) {
 			Assert.isTrue(ex instanceof UnmappableCharacterException);
-			String message= NLSUtility.format(FileBuffersMessages.ResourceTextFileBuffer_error_charset_mapping_failed_message_arg, encoding);
+			String message= NLSUtility.format(FileBuffersMessages.ResourceTextFileBuffer_error_charset_mapping_failed_message_arg, new Object[] {encoding,getLocation().toString()});
 			IStatus s= new Status(IStatus.ERROR, FileBuffersPlugin.PLUGIN_ID, IFileBufferStatusCodes.CHARSET_MAPPING_FAILED, message, ex);
 			throw new CoreException(s);
 		}
@@ -355,10 +344,10 @@ public class ResourceTextFileBuffer extends ResourceFileBuffer implements ITextF
 		 * This is a workaround for a corresponding bug in Java readers and writer,
 		 * see http://developer.java.sun.com/developer/bugParade/bugs/4508058.html
 		 */
-		if (fBOM == IContentDescription.BOM_UTF_8 && CHARSET_UTF_8.equals(encoding))
+		if (fBOM == IContentDescription.BOM_UTF_8 && StandardCharsets.UTF_8.name().equals(encoding))
 			stream= new SequenceInputStream(new ByteArrayInputStream(IContentDescription.BOM_UTF_8), stream);
 
-		if (fBOM == IContentDescription.BOM_UTF_16LE && CHARSET_UTF_16LE.equals(encoding))
+		if (fBOM == IContentDescription.BOM_UTF_16LE && StandardCharsets.UTF_16LE.name().equals(encoding))
 			stream= new SequenceInputStream(new ByteArrayInputStream(IContentDescription.BOM_UTF_16LE), stream);
 
 		if (fFile.exists()) {
@@ -513,7 +502,7 @@ public class ResourceTextFileBuffer extends ResourceFileBuffer implements ITextF
 			 * This is a workaround for a corresponding bug in Java readers and writer,
 			 * see http://developer.java.sun.com/developer/bugParade/bugs/4508058.html
 			 */
-			if (fBOM != null && CHARSET_UTF_8.equals(encoding)) {
+			if (fBOM != null && StandardCharsets.UTF_8.name().equals(encoding)) {
 				int n= 0;
 				do {
 					int bytes= contentStream.read(new byte[IContentDescription.BOM_UTF_8.length]);
@@ -527,9 +516,15 @@ public class ResourceTextFileBuffer extends ResourceFileBuffer implements ITextF
 			StringBuilder buffer= new StringBuilder(BUFFER_SIZE);
 			char[] readBuffer= new char[READER_CHUNK_SIZE];
 			int n= in.read(readBuffer);
-			while (n > 0) {
-				buffer.append(readBuffer, 0, n);
-				n= in.read(readBuffer);
+			try {
+				while (n > 0) {
+					buffer.append(readBuffer, 0, n);
+					n= in.read(readBuffer);
+				}
+			} catch (OutOfMemoryError e) {
+				// give the JVM a hint that it can free the big buffer right away
+				buffer= null;
+				throw new IOException(NLS.bind(FileBuffersMessages.ResourceTextFileBuffer_oom_on_file_read, file.getLocationURI()), e);
 			}
 
 			if (document instanceof IDocumentExtension4)
