@@ -13,19 +13,24 @@
  *******************************************************************************/
 package org.eclipse.jface.text.contentassist;
 
+import static org.eclipse.jface.util.Util.isValid;
+
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collection;
 import java.util.Collections;
-import java.util.HashSet;
+import java.util.LinkedList;
 import java.util.List;
+import java.util.Optional;
 import java.util.Set;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ExecutionException;
-import java.util.concurrent.Future;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.TimeoutException;
+import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.function.Consumer;
+import java.util.stream.Collectors;
 
 import org.eclipse.osgi.util.NLS;
 
@@ -34,7 +39,6 @@ import org.eclipse.swt.graphics.Point;
 import org.eclipse.swt.widgets.Control;
 import org.eclipse.swt.widgets.Display;
 
-import org.eclipse.core.runtime.ISafeRunnable;
 import org.eclipse.core.runtime.SafeRunner;
 
 import org.eclipse.jface.contentassist.IContentAssistSubjectControl;
@@ -55,7 +59,16 @@ import org.eclipse.jface.text.TextUtilities;
 class AsyncCompletionProposalPopup extends CompletionProposalPopup {
 
 	private static final int MAX_WAIT_IN_MS= 50; // TODO make it a preference
-	private List<CompletableFuture<List<ICompletionProposal>>> fFutures;
+
+	/**
+	 * This is only used and set when populating the dialog is async (ie computation takes more than
+	 * MAX_WAIT_IN_MS
+	 */
+	private CompletableFuture<?> fAggregatedPopulateFuture;
+
+	private Collection<CompletableFuture<?>> toCancelFutures= new LinkedList<>();
+
+	private PopupVisibleTimer fPopupVisibleTimer= new PopupVisibleTimer();
 
 	private static final class ComputingProposal implements ICompletionProposal, ICompletionProposalExtension {
 
@@ -63,7 +76,7 @@ class AsyncCompletionProposalPopup extends CompletionProposalPopup {
 		private final int fSize;
 		private int fRemaining;
 
-		public ComputingProposal(int offset, int size) {
+		ComputingProposal(int offset, int size) {
 			fSize= size;
 			fRemaining = size;
 			fOffset = offset;
@@ -122,7 +135,7 @@ class AsyncCompletionProposalPopup extends CompletionProposalPopup {
 			return -1;
 		}
 
-		public void setRemaining(int size) {
+		void setRemaining(int size) {
 			this.fRemaining = size;
 		}
 	}
@@ -147,7 +160,7 @@ class AsyncCompletionProposalPopup extends CompletionProposalPopup {
 
 		final Control control= fContentAssistSubjectControlAdapter.getControl();
 
-		if (!Helper.okToUse(fProposalShell) && control != null && !control.isDisposed()) {
+		if (!isValid(fProposalShell) && control != null && !control.isDisposed()) {
 			// add the listener before computing the proposals so we don't move the caret
 			// when the user types fast.
 			fContentAssistSubjectControlAdapter.addKeyListener(fKeyListener);
@@ -156,8 +169,7 @@ class AsyncCompletionProposalPopup extends CompletionProposalPopup {
 			fFilterOffset= fInvocationOffset;
 			fLastCompletionOffset= fFilterOffset;
 			// start invocation of processors as Futures, and make them populate the proposals upon completion
-			fFutures= buildCompletionFuturesOrJobs(fInvocationOffset);
-			runFutures(fInvocationOffset, null, true, autoActivated, true);
+			computeAndPopulateProposals(fInvocationOffset, null, true, autoActivated, true);
 		} else {
 			fLastCompletionOffset= fFilterOffset;
 			handleRepeatedInvocation();
@@ -169,94 +181,121 @@ class AsyncCompletionProposalPopup extends CompletionProposalPopup {
 	@Override
 	void handleRepeatedInvocation() {
 		cancelFutures();
-		fFutures= buildCompletionFuturesOrJobs(fInvocationOffset);
-		runFutures(fInvocationOffset, null, false, false, false);
+		computeAndPopulateProposals(fInvocationOffset, null, false, false, false);
 	}
 
-	private List<ICompletionProposal> runFutures(int offset, Consumer<List<ICompletionProposal>> callback, boolean createSelector, boolean autoActivated, boolean autoInsert) {
-		List<ICompletionProposal> computedProposals= Collections.synchronizedList(new ArrayList<>());
-		List<CompletableFuture<Void>> populateFutures= new ArrayList<>(fFutures.size());
-		for (CompletableFuture<List<ICompletionProposal>> future : fFutures) {
-			populateFutures.add(future.thenAccept(proposals -> computedProposals.addAll(proposals)));
-		}
+	private void computeAndPopulateProposals(int offset, Consumer<List<ICompletionProposal>> callback, boolean createSelector, boolean autoActivated, boolean autoInsert) {
+		List<CompletableFuture<List<ICompletionProposal>>> computationFutures= buildCompletionFuturesOrJobs(offset);
+		toCancelFutures.addAll(computationFutures);
+		fComputedProposals= Collections.synchronizedList(new ArrayList<>());
+		List<CompletableFuture<Void>> populateFutures= computationFutures.stream().map(future -> future.thenAccept(fComputedProposals::addAll)).collect(Collectors.toList());
+		toCancelFutures.addAll(populateFutures);
+		CompletableFuture<?> aggregatedPopulateFuture= CompletableFuture.allOf(populateFutures.toArray(new CompletableFuture[populateFutures.size()]));
+		toCancelFutures.add(aggregatedPopulateFuture);
 
-		long requestBeginningTimestamp= System.currentTimeMillis();
-		long stillRemainingThreeshold= MAX_WAIT_IN_MS;
-		for (CompletableFuture<?> future : populateFutures) {
-			try {
-				future.get(stillRemainingThreeshold, TimeUnit.MILLISECONDS);
-			} catch (TimeoutException | ExecutionException | InterruptedException ex) {
-				// future failed or took more time than we want to wait
-			}
-			stillRemainingThreeshold= MAX_WAIT_IN_MS - (System.currentTimeMillis() - requestBeginningTimestamp);
-			if (stillRemainingThreeshold < 0) {
-				// we already spent too much time (more than MAX_WAIT_IN_MS), stop waiting.
-				break;
-			}
+		boolean useAsyncMode= false;
+		try {
+			aggregatedPopulateFuture.get(MAX_WAIT_IN_MS, TimeUnit.MILLISECONDS);
+		} catch (TimeoutException e) {
+			useAsyncMode= true;
+		} catch (ExecutionException | InterruptedException ex) {
+			// nothing to do
 		}
-		fComputedProposals= computedProposals;
-		if (stillRemainingThreeshold > 0) { // everything ready in time, go synchronous
-			int count= computedProposals.size();
-			if (count == 0 && hideWhenNoProposals(autoActivated))
-				return computedProposals;
+		if (!useAsyncMode) {
+			int count= fComputedProposals.size();
+			if (count == 0 && hideWhenNoProposals(autoActivated)) {
+				return;
+			}
 
-			if (autoInsert && count == 1 && !autoActivated && canAutoInsert(computedProposals.get(0))) {
-				insertProposal(computedProposals.get(0), (char) 0, 0, offset);
+			if (autoInsert && count == 1 && !autoActivated &&
+					fContentAssistant.isAutoActivateCompletionOnType() && canAutoInsert(fComputedProposals.get(0))) {
+				insertProposal(fComputedProposals.get(0), (char) 0, 0, offset);
 				hide();
 			} else {
 				if (createSelector) {
 					createProposalSelector();
 				}
 				if (callback != null) {
-					callback.accept(computedProposals);
+					callback.accept(fComputedProposals);
 				} else {
-					setProposals(computedProposals, false);
+					setProposals(fComputedProposals, false);
 					displayProposals();
 				}
 			}
-		} else { // processors took too much time, go asynchronous
+		} else {
 			if (createSelector) {
 				createProposalSelector();
 			}
-			ComputingProposal computingProposal= new ComputingProposal(offset, fFutures.size());
-			computedProposals.add(0, computingProposal);
+			ComputingProposal computingProposal= new ComputingProposal(offset, populateFutures.size());
+			fComputedProposals.add(0, computingProposal);
 			setProposals(fComputedProposals, false);
-			Set<CompletableFuture<Void>> remaining= Collections.synchronizedSet(new HashSet<>(populateFutures));
-			for (CompletableFuture<Void> populateFuture : populateFutures) {
-				populateFuture.thenRun(() -> {
-					remaining.removeIf(CompletableFuture::isDone);
-					computingProposal.setRemaining(remaining.size());
-					if (remaining.isEmpty()) {
-						computedProposals.remove(computingProposal);
-					}
-					List<ICompletionProposal> newProposals= new ArrayList<>(computedProposals);
-					fComputedProposals= newProposals;
-					Display.getDefault().asyncExec(() -> {
-						if (autoInsert && !autoActivated && remaining.isEmpty() && newProposals.size() == 1 && canAutoInsert(newProposals.get(0))) {
-							if (Helper.okToUse(fProposalShell)) {
-								insertProposal(newProposals.get(0), (char) 0, 0, offset);
+			AtomicInteger remaining= new AtomicInteger(populateFutures.size());
+			final List<ICompletionProposal> requestSpecificProposals= fComputedProposals; //fComputedProposals can be changed/reset later
+			populateFutures= populateFutures.stream().map(future -> future.thenRun(() -> {
+				computingProposal.setRemaining(remaining.decrementAndGet());
+				if (remaining.get() == 0) {
+					requestSpecificProposals.remove(computingProposal);
+				}
+				Control control= fContentAssistSubjectControlAdapter.getControl();
+				if (!control.isDisposed() && offset == fInvocationOffset) {
+					control.getDisplay().asyncExec(() -> {
+						// Skip if offset has changed while runnable was scheduled
+						// nor when completion "session" was modified or canceled.
+						if (offset != fInvocationOffset || fComputedProposals != requestSpecificProposals) {
+							return;
+						}
+						if (autoInsert
+								&& !autoActivated
+								&& !fComputedProposals.contains(computingProposal)
+								&& fComputedProposals.size() == 1
+								&& remaining.get() == 0
+								&& canAutoInsert(fComputedProposals.get(0))) {
+							if (isValid(fProposalShell)) {
+								insertProposal(fComputedProposals.get(0), (char) 0, 0, offset);
 								hide();
 							}
 							return;
 						}
-						if (remaining.isEmpty() && callback != null) {
-							callback.accept(newProposals);
+						if (!fComputedProposals.contains(computingProposal) && callback != null) {
+							callback.accept(fComputedProposals);
 						} else {
-							setProposals(newProposals, false);
-							displayProposals();
+							boolean stillComputing= fComputedProposals.contains(computingProposal);
+							boolean hasProposals= (stillComputing && fComputedProposals.size() > 1)
+									|| (!stillComputing && !fComputedProposals.isEmpty());
+
+							if ((autoActivated && hasProposals) || !autoActivated) {
+								setProposals(fComputedProposals, false);
+								displayProposals(true);
+							} else if (isValid(fProposalShell) && !fProposalShell.isVisible() && remaining.get() == 0) {
+								hide(); // we only tear down if the popup is not visible.
+							}
 						}
 					});
-				});
-			}
-			displayProposals();
+				}
+			})).collect(Collectors.toList());
+			toCancelFutures.addAll(populateFutures);
+			fAggregatedPopulateFuture= CompletableFuture.allOf(populateFutures.toArray(new CompletableFuture[populateFutures.size()]));
+			toCancelFutures.add(fAggregatedPopulateFuture);
+		}
+		displayProposals(!autoActivated);
+	}
+
+	@Override
+	void displayProposals(boolean showPopup) {
+		if (showPopup) {
+			fPopupVisibleTimer.stop();
+		}
+
+		super.displayProposals(showPopup);
+		if (!showPopup) {
+			fPopupVisibleTimer.start();
 		}
-		return computedProposals;
 	}
 
 	@Override
 	public String incrementalComplete() {
 		cancelFutures();
-		if (Helper.okToUse(fProposalShell) && fFilteredProposals != null) {
+		if (isValid(fProposalShell) && fFilteredProposals != null) {
 			return super.incrementalComplete();
 		}
 		final Control control= fContentAssistSubjectControlAdapter.getControl();
@@ -264,20 +303,19 @@ class AsyncCompletionProposalPopup extends CompletionProposalPopup {
 		if (fKeyListener == null)
 			fKeyListener= new ProposalSelectionListener();
 
-		if (!Helper.okToUse(fProposalShell) && !control.isDisposed())
+		if (!isValid(fProposalShell) && !control.isDisposed())
 			fContentAssistSubjectControlAdapter.addKeyListener(fKeyListener);
 
 		fInvocationOffset= fContentAssistSubjectControlAdapter.getSelectedRange().x;
 		fFilterOffset= fInvocationOffset;
 		fLastCompletionOffset= fFilterOffset;
 
-		fFutures= buildCompletionFuturesOrJobs(fInvocationOffset);
-		fFilteredProposals= runFutures(fInvocationOffset, (List<ICompletionProposal> proposals) -> {
+		computeAndPopulateProposals(fInvocationOffset, (List<ICompletionProposal> proposals) -> {
 			ensureDocumentListenerInstalled();
-			if (proposals.size() > 0 && completeCommonPrefix()) {
+			fFilteredProposals= proposals;
+			if (!proposals.isEmpty() && completeCommonPrefix()) {
 				hide();
 			} else {
-				fFilteredProposals= proposals;
 				setProposals(proposals, false);
 				displayProposals();
 			}
@@ -301,26 +339,15 @@ class AsyncCompletionProposalPopup extends CompletionProposalPopup {
 	}
 
 	void cancelFutures() {
-		if (fFutures != null) {
-			for (Future<?> future : fFutures) {
-				future.cancel(true);
-			}
-			fFutures= null;
-		}
+		toCancelFutures.forEach(future -> future.cancel(true));
+		toCancelFutures.clear();
 	}
 
 	@Override
 	protected List<ICompletionProposal> computeFilteredProposals(int offset, DocumentEvent event) {
-		if(fComputedProposals != null && fComputedProposals.size() > 0 && fComputedProposals.get(0) instanceof ComputingProposal) {
-			Set<CompletableFuture<List<ICompletionProposal>>> remaining = Collections.synchronizedSet(new HashSet<>(fFutures));
-			for (CompletableFuture<List<ICompletionProposal>> future : fFutures) {
-				future.thenRun(() -> {
-					remaining.removeIf(CompletableFuture::isDone);
-					if (remaining.isEmpty()) {
-						filterProposals();
-					}
-				});
-			}
+		if (fAggregatedPopulateFuture != null && !fAggregatedPopulateFuture.isDone()) {
+			// user typed a char & computation still pending -> let all futures complete then invoke "filterProposals" upon completion
+			fAggregatedPopulateFuture.thenRun(this::filterProposals);
 			return fComputedProposals;
 		}
 		return super.computeFilteredProposals(offset, event);
@@ -328,6 +355,7 @@ class AsyncCompletionProposalPopup extends CompletionProposalPopup {
 
 	@Override
 	public void hide() {
+		fPopupVisibleTimer.stop();
 		super.hide();
 		cancelFutures();
 	}
@@ -346,15 +374,12 @@ class AsyncCompletionProposalPopup extends CompletionProposalPopup {
 		for (IContentAssistProcessor processor : processors) {
 			futures.add(CompletableFuture.supplyAsync(() -> {
 				AtomicReference<List<ICompletionProposal>> result= new AtomicReference<>();
-				SafeRunner.run(new ISafeRunnable() {
-					@Override
-					public void run() throws Exception {
-						ICompletionProposal[] proposals= processor.computeCompletionProposals(fViewer, invocationOffset);
-						if (proposals == null) {
-							result.set(Collections.emptyList());
-						} else {
-							result.set(Arrays.asList(proposals));
-						}
+				SafeRunner.run(() -> {
+					ICompletionProposal[] proposals= processor.computeCompletionProposals(fViewer, invocationOffset);
+					if (proposals == null) {
+						result.set(Collections.emptyList());
+					} else {
+						result.set(Arrays.asList(proposals));
 					}
 				});
 				List<ICompletionProposal> proposals= result.get();
@@ -381,4 +406,40 @@ class AsyncCompletionProposalPopup extends CompletionProposalPopup {
 		return IDocument.DEFAULT_CONTENT_TYPE;
 	}
 
+	private class PopupVisibleTimer implements Runnable {
+		private Thread fThread;
+
+		private Object fMutex= new Object();
+
+		private int fAutoActivationDelay= 500;
+
+		protected void start() {
+			fThread= new Thread(this, JFaceTextMessages.getString("ContentAssistant.assist_delay_timer_name")); //$NON-NLS-1$
+			fThread.start();
+		}
+
+		@Override
+		public void run() {
+			try {
+				while (true) {
+					synchronized (fMutex) {
+						if (fAutoActivationDelay != 0)
+							fMutex.wait(fAutoActivationDelay);
+					}
+					Optional<Display> display= Optional.ofNullable(fContentAssistSubjectControlAdapter.getControl()).map(Control::getDisplay);
+					display.ifPresent(d -> d.asyncExec(() -> displayProposals(true)));
+					break;
+				}
+			} catch (InterruptedException e) {
+			}
+			fThread= null;
+		}
+
+		protected void stop() {
+			Thread threadToStop= fThread;
+			if (threadToStop != null && threadToStop.isAlive())
+				threadToStop.interrupt();
+		}
+
+	}
 }
