@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2018 IBM Corporation and others.
+ * Copyright (c) 2000, 2022 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -32,6 +32,11 @@ import org.eclipse.swt.widgets.Shell;
 
 import org.eclipse.core.runtime.Assert;
 
+import org.eclipse.text.undo.DocumentUndoEvent;
+import org.eclipse.text.undo.DocumentUndoManagerRegistry;
+import org.eclipse.text.undo.IDocumentUndoListener;
+import org.eclipse.text.undo.IDocumentUndoManager;
+
 import org.eclipse.jface.internal.text.link.contentassist.ContentAssistant2;
 import org.eclipse.jface.internal.text.link.contentassist.IProposalListener;
 import org.eclipse.jface.viewers.IPostSelectionProvider;
@@ -277,6 +282,19 @@ public class LinkedModeUI {
 		 *         should be taken
 		 */
 		ExitFlags doExit(LinkedModeModel model, VerifyEvent event, int offset, int length);
+
+		/**
+		 * Checks whether the linked mode should be left after receiving the given
+		 * <code>DocumentEvent</code>, especially allowing to control Copy-Paste operations.
+		 *
+		 * @param model the linked mode model
+		 * @param event the document event
+		 * @return valid exit flags or <code>null</code> if no special action should be taken
+		 * @since 3.22
+		 */
+		default ExitFlags doExit(LinkedModeModel model, DocumentEvent event) {
+			return null;
+		}
 	}
 
 	/**
@@ -292,7 +310,7 @@ public class LinkedModeUI {
 	/**
 	 * Listens for shell events and acts upon them.
 	 */
-	private class Closer implements ShellListener, ITextInputListener {
+	private class Closer implements ShellListener, ITextInputListener, IDocumentUndoListener {
 
 		@Override
 		public void shellActivated(ShellEvent e) {
@@ -369,6 +387,34 @@ public class LinkedModeUI {
 		public void inputDocumentChanged(IDocument oldInput, IDocument newInput) {
 		}
 
+		@Override
+		public void documentUndoNotification(DocumentUndoEvent event) {
+			int type= event.getEventType();
+			if ((type & DocumentUndoEvent.ABOUT_TO_UNDO) != 0 || (type & DocumentUndoEvent.ABOUT_TO_REDO) != 0) {
+				// default behavior: any document change outside a linked position
+				// causes us to exit
+				String textRemoved= event.getPreservedText();
+				int end= event.getOffset() + (textRemoved != null ? textRemoved.length() : 0);
+				for (int offset= event.getOffset(); offset <= end; offset++) {
+					if (!fModel.anyPositionContains(offset)) {
+						ITextViewer viewer= fCurrentTarget.getViewer();
+						if (fFramePosition != null && viewer instanceof IEditingSupportRegistry) {
+							IEditingSupport[] helpers= ((IEditingSupportRegistry) viewer).getRegisteredSupports();
+							for (IEditingSupport helper : helpers) {
+								if (helper.isOriginator(null, new Region(fFramePosition.getOffset(), fFramePosition.getLength())))
+									return;
+							}
+						}
+
+						leave(ILinkedModeListener.EXTERNAL_MODIFICATION);
+						return;
+					}
+				}
+
+				// Make sure that any document compound change is done committed
+				endCompoundChangeIfNeeded();
+			}
+		}
 	}
 
 	/**
@@ -397,6 +443,15 @@ public class LinkedModeUI {
 				}
 			}
 
+			// Apply  ExitPolicy to any inserted text if the insertion is made inside a linked region
+			if (fExitPolicy != null) {
+				ExitFlags flags= fExitPolicy.doExit(fModel, event);
+				if (flags != null) {
+					leave(flags.flags);
+					return;
+				}
+			}
+
 			// Make sure that any document change is done inside a compound change
 			beginCompoundChangeIfNeeded();
 
@@ -955,6 +1010,11 @@ public class LinkedModeUI {
 		viewer.addTextInputListener(fCloser);
 
 		viewer.getDocument().addDocumentListener(fDocumentListener);
+
+		IDocumentUndoManager undoManager= DocumentUndoManagerRegistry.getDocumentUndoManager(viewer.getDocument());
+		if (undoManager != null) {
+			undoManager.addDocumentUndoListener(fCloser);
+		}
 	}
 
 	/**
@@ -1097,6 +1157,11 @@ public class LinkedModeUI {
 
 		((IPostSelectionProvider) viewer).removePostSelectionChangedListener(fSelectionListener);
 
+		IDocumentUndoManager undoManager= DocumentUndoManagerRegistry.getDocumentUndoManager(viewer.getDocument());
+		if (undoManager != null) {
+			undoManager.removeDocumentUndoListener(fCloser);
+		}
+
 		redraw();
 	}
 
