@@ -113,6 +113,7 @@ abstract public class AbstractReconciler implements IReconciler {
 		 * emptied the dirty region queue.
 		 */
 		public void suspendCallerWhileDirty() {
+			AbstractReconciler.this.signalWaitForFinish();
 			boolean isDirty;
 			do {
 				synchronized (fDirtyRegionQueue) {
@@ -138,6 +139,9 @@ abstract public class AbstractReconciler implements IReconciler {
 					fIsDirty= true;
 					fReset= true;
 				}
+				synchronized (fDirtyRegionQueue) {
+					fDirtyRegionQueue.notifyAll(); // wake up wait(fDelay);
+				}
 
 			} else {
 
@@ -150,6 +154,7 @@ abstract public class AbstractReconciler implements IReconciler {
 				}
 			}
 
+			informNotFinished();
 			reconcilerReset();
 		}
 
@@ -164,12 +169,7 @@ abstract public class AbstractReconciler implements IReconciler {
 		@Override
 		public void run() {
 
-			synchronized (fDirtyRegionQueue) {
-				try {
-					fDirtyRegionQueue.wait(fDelay);
-				} catch (InterruptedException x) {
-				}
-			}
+			delay();
 
 			if (fCanceled)
 				return;
@@ -178,18 +178,15 @@ abstract public class AbstractReconciler implements IReconciler {
 
 			while (!fCanceled) {
 
-				synchronized (fDirtyRegionQueue) {
-					try {
-						fDirtyRegionQueue.wait(fDelay);
-					} catch (InterruptedException x) {
-					}
-				}
+				delay();
 
 				if (fCanceled)
 					break;
 
-				if (!isDirty())
+				if (!isDirty()) {
+					waitFinish= false; //signalWaitForFinish() was called but nothing todo
 					continue;
+				}
 
 				synchronized (this) {
 					if (fReset) {
@@ -238,7 +235,7 @@ abstract public class AbstractReconciler implements IReconciler {
 			if (fThread.isActive() || !fThread.isDirty() && fThread.isAlive()) {
 				if (!fIsAllowedToModifyDocument && Thread.currentThread() == fThread)
 					throw new UnsupportedOperationException("The reconciler thread is not allowed to modify the document"); //$NON-NLS-1$
-				aboutToBeReconciled();
+				aboutToBeReconciledInternal();
 			}
 
 			/*
@@ -292,7 +289,7 @@ abstract public class AbstractReconciler implements IReconciler {
 			fDocument.addDocumentListener(this);
 
 			if (!fThread.isDirty())
-				aboutToBeReconciled();
+				aboutToBeReconciledInternal();
 
 			startReconciling();
 		}
@@ -306,6 +303,8 @@ abstract public class AbstractReconciler implements IReconciler {
 	private Listener fListener;
 	/** The background thread delay. */
 	private int fDelay= 500;
+	/** Signal that the the background thread should not delay. */
+	volatile boolean waitFinish;
 	/** Are there incremental reconciling strategies? */
 	private boolean fIsIncrementalReconciler= true;
 	/** The progress monitor used by this reconciler. */
@@ -502,7 +501,7 @@ abstract public class AbstractReconciler implements IReconciler {
 				// Insert
 				fDirtyRegionQueue.addDirtyRegion(new DirtyRegion(e.getOffset(), e.getText().length(), DirtyRegion.INSERT, e.getText()));
 
-			} else if (e.getText() == null || e.getText().length() == 0) {
+			} else if (e.getText() == null || e.getText().isEmpty()) {
 				// Remove
 				fDirtyRegionQueue.addDirtyRegion(new DirtyRegion(e.getOffset(), e.getLength(), DirtyRegion.REMOVE, null));
 
@@ -526,6 +525,56 @@ abstract public class AbstractReconciler implements IReconciler {
 	protected void aboutToBeReconciled() {
 	}
 
+	/**
+	 * Hook for subclasses which want to perform some action as soon as the reconciler starts work
+	 * (initial or reconciling) or waiting.
+	 * <p>
+	 * Default implementation is to do nothing. Implementors may call
+	 * {@link #signalWaitForFinish()}.
+	 * </p>
+	 *
+	 * @since 3.20
+	 * @see #signalWaitForFinish
+	 */
+	protected void aboutToWork() {
+	}
+
+	/**
+	 * Signal reconciling should finish as soon as possible.
+	 *
+	 * @since 3.20
+	 * @see #aboutToWork
+	 */
+	public void signalWaitForFinish() {
+		synchronized (fDirtyRegionQueue) {
+			waitFinish= true;
+			fDirtyRegionQueue.notifyAll(); // notify AbstractReconciler#delay about waitFinish
+		}
+	}
+
+	private void informNotFinished() {
+		waitFinish= false;
+		aboutToWork();
+	}
+
+	private void aboutToBeReconciledInternal() {
+		aboutToBeReconciled();
+		informNotFinished();
+	}
+
+
+	private void delay() {
+		synchronized (fDirtyRegionQueue) {
+			if (waitFinish) {
+				return; // do not delay when waiting;
+			}
+			try {
+				fDirtyRegionQueue.wait(fDelay);
+			} catch (InterruptedException x) {
+			}
+		}
+	}
+
 	/**
 	 * This method is called on startup of the background activity. It is called only
 	 * once during the life time of the reconciler. Clients may reimplement this method.
@@ -542,7 +591,7 @@ abstract public class AbstractReconciler implements IReconciler {
 		if (fDocument != null) {
 
 			if (!fThread.isDirty()&& fThread.isAlive())
-				aboutToBeReconciled();
+				aboutToBeReconciledInternal();
 
 			if (fThread.isActive())
 				fProgressMonitor.setCanceled(true);
