@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2018 IBM Corporation and others.
+ * Copyright (c) 2000, 2019 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -20,11 +20,11 @@ import java.nio.charset.CharsetEncoder;
 import java.nio.charset.IllegalCharsetNameException;
 import java.nio.charset.StandardCharsets;
 import java.nio.charset.UnsupportedCharsetException;
+import java.text.BreakIterator;
+import java.text.DateFormat;
+import java.text.MessageFormat;
+import java.util.Date;
 import java.util.Iterator;
-import java.util.List;
-
-import com.ibm.icu.text.BreakIterator;
-import com.ibm.icu.text.MessageFormat;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.custom.StyledText;
@@ -121,6 +121,7 @@ import org.eclipse.ui.IEditorDescriptor;
 import org.eclipse.ui.IEditorInput;
 import org.eclipse.ui.IEditorPart;
 import org.eclipse.ui.IFileEditorInput;
+import org.eclipse.ui.IPageLayout;
 import org.eclipse.ui.IURIEditorInput;
 import org.eclipse.ui.IWorkbenchActionConstants;
 import org.eclipse.ui.IWorkbenchCommandConstants;
@@ -151,6 +152,7 @@ import org.eclipse.ui.keys.IBindingService;
 import org.eclipse.ui.operations.NonLocalUndoUserApprover;
 import org.eclipse.ui.part.FileEditorInput;
 import org.eclipse.ui.part.IShowInSource;
+import org.eclipse.ui.part.IShowInTargetList;
 import org.eclipse.ui.part.ShowInContext;
 import org.eclipse.ui.views.markers.MarkerViewUtil;
 
@@ -224,7 +226,7 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 
 	/**
 	 * Preference key that controls whether to use saturated colors in the overview ruler.
-	 * 
+	 *
 	 * @since 3.8
 	 */
 	private static final String USE_SATURATED_COLORS_IN_OVERVIEW_RULER= AbstractDecoratedTextEditorPreferenceConstants.USE_SATURATED_COLORS_IN_OVERVIEW_RULER;
@@ -312,7 +314,7 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 	 * @since 3.6
 	 */
 	private boolean fIsComingFromGotoMarker= false;
-	
+
 	/**
 	 * Tells whether editing the current derived editor input is allowed.
 	 * @since 3.3
@@ -340,7 +342,7 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 
 	/**
 	 * Creates a new text editor.
-	 * 
+	 *
 	 * @see #initializeEditor()
 	 * @see #initializeKeyBindingScopes()
 	 */
@@ -509,16 +511,13 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 	@Override
 	protected IMenuListener createContextMenuListener() {
 		final IMenuListener superListener= super.createContextMenuListener();
-		return new IMenuListener() {
-			@Override
-			public void menuAboutToShow(IMenuManager menu) {
-				if (!getOverviewRulerContextMenuId().equals(menu.getId())) {
-					superListener.menuAboutToShow(menu);
-					return;
-				}
-				setFocus();
-				overviewRulerContextMenuAboutToShow(menu);
+		return menu -> {
+			if (!getOverviewRulerContextMenuId().equals(menu.getId())) {
+				superListener.menuAboutToShow(menu);
+				return;
 			}
+			setFocus();
+			overviewRulerContextMenuAboutToShow(menu);
 		};
 	}
 
@@ -653,7 +652,7 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 
 	/**
 	 * Checks if the preference to use saturated colors is enabled for the overview ruler.
-	 * 
+	 *
 	 * @return <code>true</code> if the saturated colors preference is enabled, <code>false</code>
 	 *         otherwise
 	 * @since 3.8
@@ -854,7 +853,8 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 				return;
 			}
 
-			if (AbstractDecoratedTextEditorPreferenceConstants.EDITOR_SPACES_FOR_TABS.equals(property)) {
+			if (AbstractDecoratedTextEditorPreferenceConstants.EDITOR_SPACES_FOR_TABS.equals(property)
+					|| AbstractDecoratedTextEditorPreferenceConstants.EDITOR_DELETE_SPACES_AS_TABS.equals(property)) {
 				if (isTabsToSpacesConversionEnabled())
 					installTabsToSpacesConverter();
 				else
@@ -880,8 +880,8 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 
 			if (sourceViewer instanceof ITextViewerExtension6) {
 				HyperlinkDetectorDescriptor[] descriptor= EditorsUI.getHyperlinkDetectorRegistry().getHyperlinkDetectorDescriptors();
-				for (int i= 0; i < descriptor.length; i++) {
-					if (descriptor[i].getId().equals(property) || (descriptor[i].getId() + HyperlinkDetectorDescriptor.STATE_MASK_POSTFIX).equals(property)) {
+				for (HyperlinkDetectorDescriptor d : descriptor) {
+					if (d.getId().equals(property) || (d.getId() + HyperlinkDetectorDescriptor.STATE_MASK_POSTFIX).equals(property)) {
 						IHyperlinkDetector[] detectors= getSourceViewerConfiguration().getHyperlinkDetectors(sourceViewer);
 						int stateMask= getSourceViewerConfiguration().getHyperlinkStateMask(sourceViewer);
 						ITextViewerExtension6 textViewer6= (ITextViewerExtension6)sourceViewer;
@@ -959,7 +959,7 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 		if (fSourceViewerDecorationSupport == null) {
 			fSourceViewerDecorationSupport= new SourceViewerDecorationSupport(viewer, getOverviewRuler(), getAnnotationAccess(), getSharedColors());
 			configureSourceViewerDecorationSupport(fSourceViewerDecorationSupport);
-			
+
 			// Fix for overridden print margin column, see https://bugs.eclipse.org/468307
 			if (!getPreferenceStore().getBoolean(PRINT_MARGIN_ALLOW_OVERRIDE))
 				fSourceViewerDecorationSupport.setMarginPainterPreferenceKeys(PRINT_MARGIN, PRINT_MARGIN_COLOR, PRINT_MARGIN_COLUMN);
@@ -1113,9 +1113,10 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 			return !isReadOnlyLocalStatus(status);
 
 		IStatus[] childrenStatus= status.getChildren();
-		for (int i= 0; i < childrenStatus.length; i++) {
-			if (childrenStatus[i].getSeverity() == IStatus.ERROR && !isReadOnlyLocalStatus(childrenStatus[i]))
+		for (IStatus childrenStatu : childrenStatus) {
+			if (childrenStatu.getSeverity() == IStatus.ERROR && !isReadOnlyLocalStatus(childrenStatu)) {
 				return true;
+			}
 		}
 
 		return false;
@@ -1264,7 +1265,7 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 		// Override print action to provide additional options
 		if (getAction(ITextEditorActionConstants.PRINT).isEnabled() && getSourceViewer() instanceof ITextViewerExtension8)
 			createPrintAction();
-		
+
 		action= new ResourceAction(TextEditorMessages.getBundleForConstructedKeys(), "Editor.ShowChangeRulerInformation.", IAction.AS_PUSH_BUTTON) { //$NON-NLS-1$
 			@Override
 			public void run() {
@@ -1287,27 +1288,27 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 	/**
 	 * Opens a sticky change ruler hover for the caret line. Does nothing if no change hover is
 	 * available.
-	 * 
+	 *
 	 * @since 3.5
 	 */
 	private void showChangeRulerInformation() {
 		IVerticalRuler ruler= getVerticalRuler();
 		if (!(ruler instanceof CompositeRuler) || fLineColumn == null)
 			return;
-		
+
 		CompositeRuler compositeRuler= (CompositeRuler)ruler;
 
 		// fake a mouse move (some hovers rely on this to determine the hovered line):
 		int x= fLineColumn.getControl().getLocation().x;
-		
+
 		ISourceViewer sourceViewer= getSourceViewer();
 		StyledText textWidget= sourceViewer.getTextWidget();
 		int caretOffset= textWidget.getCaretOffset();
 		int caretLine= textWidget.getLineAtOffset(caretOffset);
 		int y= textWidget.getLinePixel(caretLine);
-		
+
 		compositeRuler.setLocationOfLastMouseButtonActivity(x, y);
-		
+
 		IAnnotationHover hover= fLineColumn.getHover();
 		showFocusedRulerHover(hover, sourceViewer, caretOffset);
 	}
@@ -1315,39 +1316,39 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 	/**
 	 * Opens a sticky annotation ruler hover for the caret line. Does nothing if no annotation hover
 	 * is available.
-	 * 
+	 *
 	 * @since 3.6
 	 */
 	private void showRulerAnnotationInformation() {
 		ISourceViewer sourceViewer= getSourceViewer();
 		IAnnotationHover hover= getSourceViewerConfiguration().getAnnotationHover(sourceViewer);
 		int caretOffset= sourceViewer.getTextWidget().getCaretOffset();
-		
+
 		showFocusedRulerHover(hover, sourceViewer, caretOffset);
 	}
 
 	/**
 	 * Shows a focused hover at the specified offset.
 	 * Does nothing if <code>hover</code> is <code>null</code> or cannot be shown.
-	 * 
+	 *
 	 * @param hover the hover to be shown, can be <code>null</code>
 	 * @param sourceViewer the source viewer
 	 * @param caretOffset the caret offset
-	 * 
+	 *
 	 * @since 3.6
 	 */
 	private void showFocusedRulerHover(IAnnotationHover hover, ISourceViewer sourceViewer, int caretOffset) {
 		if (hover == null)
 			return;
-		
+
 		int modelCaretOffset= widgetOffset2ModelOffset(sourceViewer, caretOffset);
 		if (modelCaretOffset == -1)
 			return;
-		
+
 		IDocument document= sourceViewer.getDocument();
 		if (document == null)
 			return;
-		
+
 		try {
 			int line= document.getLineOfOffset(modelCaretOffset);
 			if (fInformationPresenter == null) {
@@ -1375,7 +1376,8 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 				options.printTextForeground= true;
 				options.printTextBackground= true;
 				options.jobName= getTitle();
-				options.header= StyledTextPrintOptions.SEPARATOR + getTitle();
+				options.header= getTitle() + StyledTextPrintOptions.SEPARATOR + StyledTextPrintOptions.SEPARATOR
+						+ DateFormat.getDateTimeInstance(DateFormat.FULL, DateFormat.SHORT).format(new Date());
 				options.footer= StyledTextPrintOptions.SEPARATOR + NLSUtility.format(TextEditorMessages.AbstractDecoratedTextEditor_printPageNumber, StyledTextPrintOptions.PAGE_TAG);
 
 				if (isLineNumberRulerVisible()) {
@@ -1406,15 +1408,12 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 			return (T) getAnnotationAccess();
 
 		if (adapter == IShowInSource.class) {
-			return (T) new IShowInSource() {
-				@Override
-				public ShowInContext getShowInContext() {
-					ISelection selection= null;
-					ISelectionProvider selectionProvider= getSelectionProvider();
-					if (selectionProvider != null)
-						selection= selectionProvider.getSelection();
-					return new ShowInContext(getEditorInput(), selection);
-				}
+			return (T) (IShowInSource) () -> {
+				ISelection selection= null;
+				ISelectionProvider selectionProvider= getSelectionProvider();
+				if (selectionProvider != null)
+					selection= selectionProvider.getSelection();
+				return new ShowInContext(getEditorInput(), selection);
 			};
 		}
 
@@ -1426,6 +1425,9 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 		if (MarkerAnnotationPreferences.class.equals(adapter))
 			return (T) EditorsPlugin.getDefault().getMarkerAnnotationPreferences();
 
+		if (IShowInTargetList.class.equals(adapter))
+			return (T) getShowInTargetList();
+
 		return super.getAdapter(adapter);
 
 	}
@@ -1632,7 +1634,7 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 	 * Overrides the default behavior by showing a more advanced error dialog in case of encoding
 	 * problems.
 	 * </p>
-	 * 
+	 *
 	 * @param title the dialog title
 	 * @param message the message to display
 	 * @param exception the exception to handle
@@ -1650,7 +1652,7 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 		final int saveAsUTF8ButtonId= IDialogConstants.OK_ID + IDialogConstants.CANCEL_ID + 1;
 		final int selectUnmappableCharButtonId= saveAsUTF8ButtonId + 1;
 		final Charset charset= getCharset();
-		
+
 		ErrorDialog errorDialog= new ErrorDialog(getSite().getShell(), title, message, status, IStatus.ERROR) {
 
 			@Override
@@ -1714,7 +1716,7 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 
 	/**
 	 * Returns the charset of the current editor input.
-	 * 
+	 *
 	 * @return the charset of the current editor input or <code>null</code> if it fails
 	 * @since 3.6
 	 */
@@ -1928,13 +1930,11 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 		final IColumnSupport support= getAdapter(IColumnSupport.class);
 		IPreferenceStore store= EditorsUI.getPreferenceStore();
 		final RulerColumnPreferenceAdapter adapter= new RulerColumnPreferenceAdapter(store, AbstractTextEditor.PREFERENCE_RULER_CONTRIBUTIONS);
-		List<RulerColumnDescriptor> descriptors= RulerColumnRegistry.getDefault().getColumnDescriptors();
-		for (Iterator<RulerColumnDescriptor> t= descriptors.iterator(); t.hasNext();) {
-			final RulerColumnDescriptor descriptor= t.next();
+		for (RulerColumnDescriptor descriptor : RulerColumnRegistry.getDefault().getColumnDescriptors()) {
 			if (!descriptor.isIncludedInMenu() || !support.isColumnSupported(descriptor))
 				continue;
 			final boolean isVisible= support.isColumnVisible(descriptor);
-			IAction action= new Action(MessageFormat.format(TextEditorMessages.AbstractDecoratedTextEditor_show_ruler_label, new Object[] {descriptor.getName()}), IAction.AS_CHECK_BOX) {
+			IAction action= new Action(MessageFormat.format(TextEditorMessages.AbstractDecoratedTextEditor_show_ruler_label, descriptor.getName()), IAction.AS_CHECK_BOX) {
 				@Override
 				public void run() {
 					if (descriptor.isGlobal())
@@ -2016,7 +2016,7 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 
 	/**
 	 * Selects and reveals the given offset and length in the given editor part.
-	 * 
+	 *
 	 * @param editor the editor part
 	 * @param offset the offset
 	 * @param length the length
@@ -2086,13 +2086,36 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 		return NLSUtility.format(TextEditorMessages.AbstractDecoratedTextEditor_showIn_menu, keyBinding);
 	}
 
+	/**
+	 * Creates and returns the list of target part IDs for the Show In menu
+	 *
+	 * @return the 'Show In' target part IDs
+	 * @since 3.13
+	 */
+	protected String[] createShowInTargetList() {
+		return new String[] { IPageLayout.ID_MINIMAP_VIEW };
+	}
+
+	/**
+	 * Returns the Show In target list
+	 *
+	 * @return the IShowInTargetList adapter, or <code>null</code> if no targets are listed
+	 */
+	private IShowInTargetList getShowInTargetList() {
+		final String[] targetList= createShowInTargetList();
+		if (targetList != null && targetList.length > 0) {
+			return () -> targetList;
+		}
+		return null;
+	}
+
 	/**
 	 * Returns the preference page ids of the preference pages to be shown when executing the
 	 * preferences action from the editor context menu. The first page will be selected.
 	 * <p>
 	 * Subclasses may extend or replace.
 	 * </p>
-	 * 
+	 *
 	 * @return the preference page ids to show, may be empty
 	 * @since 3.1
 	 */
@@ -2116,7 +2139,7 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 	 * <p>
 	 * Subclasses may extend or replace.
 	 * </p>
-	 * 
+	 *
 	 * @return the preference page ids to show, may be empty
 	 * @since 3.1
 	 */
@@ -2134,7 +2157,7 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 	 * <p>
 	 * Subclasses may extend or replace.
 	 * </p>
-	 * 
+	 *
 	 * @return the preference page ids to show, may be empty
 	 * @since 3.4
 	 */
@@ -2231,4 +2254,12 @@ public abstract class AbstractDecoratedTextEditor extends StatusTextEditor {
 	protected boolean isTabsToSpacesConversionEnabled() {
 		return getPreferenceStore() != null && getPreferenceStore().getBoolean(AbstractDecoratedTextEditorPreferenceConstants.EDITOR_SPACES_FOR_TABS);
 	}
+
+	/**
+	 * @since 3.13
+	 */
+	@Override
+	protected boolean isSpacesAsTabsDeletionEnabled() {
+		return getPreferenceStore() != null && getPreferenceStore().getBoolean(AbstractDecoratedTextEditorPreferenceConstants.EDITOR_DELETE_SPACES_AS_TABS);
+	}
 }
