@@ -40,7 +40,7 @@ import org.eclipse.ui.internal.texteditor.TextEditorPlugin;
  * that can be used to obtain the currently set value from an
  * <code>IPreferenceStore</code>.
  * </p>
- * <h3>The following annotation preferences are covered:</h4>
+ * <h3>The following annotation preferences are covered:</h3>
  * <h4>Display Preferences controlling how and where annotations are shown</h4>
  * <ul>
  * <li>as text background highlighting (and respective preference key),</li>
@@ -48,11 +48,11 @@ import org.eclipse.ui.internal.texteditor.TextEditorPlugin;
  * <li>on the overview ruler (and respective preference key),</li>
  * <li>on the overview ruler header,</li>
  * <li>on the vertical ruler (and respective preference key),</li>
- * </li>
  * </ul>
  * <h4>Additional Display Preferences</h4>
  * <ul>
- * <li>the base color for annotations of this type (and respective preference key),
+ * <li>the base color for annotations of this type (and respective preference
+ * key),
  * <li>the style of text decorations that are drawn (and respective preference
  * key),</li>
  * <li>the annotation image provider,</li>
@@ -847,9 +847,9 @@ public class AnnotationPreference {
 	}
 
 	/**
- 	 * Sets the preference key for go to previous navigation enablement.
- 	 *
- 	 * @param isGoToPreviousNavigationTargetKey the preference key
+	 * Sets the preference key for go to previous navigation enablement.
+	 *
+	 * @param isGoToPreviousNavigationTargetKey the preference key
 	 * @since 3.0
 	 */
 	public void setIsGoToPreviousNavigationTargetKey(String isGoToPreviousNavigationTargetKey) {
@@ -1072,9 +1072,9 @@ public class AnnotationPreference {
 		if (!getAnnotationType().equals(preference.getAnnotationType()))
 			return;
 
-		for (int i= 0; i < ATTRIBUTES.length; i++) {
-			if (!hasValue(ATTRIBUTES[i]))
-				setValue(ATTRIBUTES[i], preference.getValue(ATTRIBUTES[i]));
+		for (Object element : ATTRIBUTES) {
+			if (!hasValue(element))
+				setValue(element, preference.getValue(element));
 		}
 
 		if (fAnnotationImageProvider == null)
