@@ -23,6 +23,7 @@ import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
+import java.text.Collator;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
@@ -32,7 +33,7 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 
-import com.ibm.icu.text.Collator;
+import org.osgi.framework.FrameworkUtil;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.custom.StyledText;
@@ -68,7 +69,6 @@ import org.eclipse.core.runtime.Status;
 import org.eclipse.text.templates.TemplatePersistenceData;
 import org.eclipse.text.templates.TemplateReaderWriter;
 
-import org.eclipse.jface.action.Action;
 import org.eclipse.jface.action.GroupMarker;
 import org.eclipse.jface.action.IAction;
 import org.eclipse.jface.action.IMenuManager;
@@ -91,6 +91,7 @@ import org.eclipse.jface.viewers.IBaseLabelProvider;
 import org.eclipse.jface.viewers.IStructuredSelection;
 import org.eclipse.jface.viewers.ITableLabelProvider;
 import org.eclipse.jface.viewers.LabelProvider;
+import org.eclipse.jface.viewers.SelectionChangedEvent;
 import org.eclipse.jface.viewers.StructuredSelection;
 import org.eclipse.jface.viewers.TableViewer;
 import org.eclipse.jface.viewers.Viewer;
@@ -100,7 +101,6 @@ import org.eclipse.jface.window.Window;
 import org.eclipse.jface.text.Document;
 import org.eclipse.jface.text.IDocument;
 import org.eclipse.jface.text.ITextOperationTarget;
-import org.eclipse.jface.text.ITextViewer;
 import org.eclipse.jface.text.contentassist.ContentAssistant;
 import org.eclipse.jface.text.contentassist.IContentAssistProcessor;
 import org.eclipse.jface.text.contentassist.IContentAssistant;
@@ -118,11 +118,13 @@ import org.eclipse.ui.IWorkbench;
 import org.eclipse.ui.IWorkbenchCommandConstants;
 import org.eclipse.ui.IWorkbenchPreferencePage;
 import org.eclipse.ui.PlatformUI;
+import org.eclipse.ui.actions.ActionFactory;
 import org.eclipse.ui.handlers.IHandlerActivation;
 import org.eclipse.ui.handlers.IHandlerService;
 import org.eclipse.ui.internal.texteditor.NLSUtility;
 import org.eclipse.ui.internal.texteditor.SWTUtil;
 import org.eclipse.ui.internal.texteditor.TextEditorPlugin;
+import org.eclipse.ui.internal.texteditor.templates.TextViewerAction;
 
 import org.eclipse.ui.texteditor.ITextEditorActionConstants;
 import org.eclipse.ui.texteditor.ITextEditorActionDefinitionIds;
@@ -152,55 +154,6 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 	 */
 	protected static class EditTemplateDialog extends StatusDialog {
 
-		private class TextViewerAction extends Action implements IUpdate {
-
-			private int fOperationCode= -1;
-			private ITextOperationTarget fOperationTarget;
-
-			/**
-			 * Creates a new action.
-			 *
-			 * @param viewer the viewer
-			 * @param operationCode the opcode
-			 */
-			public TextViewerAction(ITextViewer viewer, int operationCode) {
-				fOperationCode= operationCode;
-				fOperationTarget= viewer.getTextOperationTarget();
-				update();
-			}
-
-			/**
-			 * Updates the enabled state of the action.
-			 * Fires a property change if the enabled state changes.
-			 *
-			 * @see Action#firePropertyChange(String, Object, Object)
-			 */
-			@Override
-			public void update() {
-				// XXX: workaround for https://bugs.eclipse.org/bugs/show_bug.cgi?id=206111
-				if (fOperationCode == ITextOperationTarget.REDO || fOperationCode == ITextOperationTarget.REDO)
-					return;
-
-				boolean wasEnabled= isEnabled();
-				boolean isEnabled= (fOperationTarget != null && fOperationTarget.canDoOperation(fOperationCode));
-				setEnabled(isEnabled);
-
-				if (wasEnabled != isEnabled) {
-					firePropertyChange(ENABLED, wasEnabled ? Boolean.TRUE : Boolean.FALSE, isEnabled ? Boolean.TRUE : Boolean.FALSE);
-				}
-			}
-
-			/**
-			 * @see Action#run()
-			 */
-			@Override
-			public void run() {
-				if (fOperationCode != -1 && fOperationTarget != null) {
-					fOperationTarget.doOperation(fOperationCode);
-				}
-			}
-		}
-
 		private final Template fOriginalTemplate;
 
 		private Text fNameText;
@@ -275,7 +228,7 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 		public void create() {
 			super.create();
 			// update initial OK button to be disabled for new templates
-			boolean valid= fNameText == null || fNameText.getText().trim().length() != 0;
+			boolean valid= fNameText == null || !fNameText.getText().trim().isEmpty();
 			if (!valid) {
 				StatusInfo status = new StatusInfo();
 				status.setError(TemplatesMessages.EditTemplateDialog_error_noname);
@@ -329,8 +282,8 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 				createLabel(composite, TemplatesMessages.EditTemplateDialog_context);
 				fContextCombo= new Combo(composite, SWT.READ_ONLY);
 
-				for (int i= 0; i < fContextTypes.length; i++) {
-					fContextCombo.add(fContextTypes[i][1]);
+				for (String[] fContextType : fContextTypes) {
+					fContextCombo.add(fContextType[1]);
 				}
 
 				fContextCombo.addModifyListener(listener);
@@ -406,9 +359,9 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 		private String getContextId() {
 			if (fContextCombo != null && !fContextCombo.isDisposed()) {
 				String name= fContextCombo.getText();
-				for (int i= 0; i < fContextTypes.length; i++) {
-					if (name.equals(fContextTypes[i][1])) {
-						return fContextTypes[i][0];
+				for (String[] fContextType : fContextTypes) {
+					if (name.equals(fContextType[1])) {
+						return fContextType[0];
 					}
 				}
 			}
@@ -552,26 +505,32 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 			});
 
 			TextViewerAction action= new TextViewerAction(fPatternEditor, ITextOperationTarget.UNDO);
+			action.setActionDefinitionId(ActionFactory.UNDO.getCommandId());
 			action.setText(TemplatesMessages.EditTemplateDialog_undo);
 			fGlobalActions.put(ITextEditorActionConstants.UNDO, action);
 
 			action= new TextViewerAction(fPatternEditor, ITextOperationTarget.REDO);
+			action.setActionDefinitionId(ActionFactory.REDO.getCommandId());
 			action.setText(TemplatesMessages.EditTemplateDialog_redo);
 			fGlobalActions.put(ITextEditorActionConstants.REDO, action);
 
-			action= new TextViewerAction(fPatternEditor, ITextOperationTarget.CUT);
+			action = new TextViewerAction(fPatternEditor, ITextOperationTarget.CUT);
+			action.setActionDefinitionId(ActionFactory.CUT.getCommandId());
 			action.setText(TemplatesMessages.EditTemplateDialog_cut);
 			fGlobalActions.put(ITextEditorActionConstants.CUT, action);
 
-			action= new TextViewerAction(fPatternEditor, ITextOperationTarget.COPY);
+			action = new TextViewerAction(fPatternEditor, ITextOperationTarget.COPY);
+			action.setActionDefinitionId(ActionFactory.COPY.getCommandId());
 			action.setText(TemplatesMessages.EditTemplateDialog_copy);
 			fGlobalActions.put(ITextEditorActionConstants.COPY, action);
 
 			action= new TextViewerAction(fPatternEditor, ITextOperationTarget.PASTE);
+			action.setActionDefinitionId(ActionFactory.PASTE.getCommandId());
 			action.setText(TemplatesMessages.EditTemplateDialog_paste);
 			fGlobalActions.put(ITextEditorActionConstants.PASTE, action);
 
 			action= new TextViewerAction(fPatternEditor, ITextOperationTarget.SELECT_ALL);
+			action.setActionDefinitionId(ActionFactory.SELECT_ALL.getCommandId());
 			action.setText(TemplatesMessages.EditTemplateDialog_select_all);
 			fGlobalActions.put(ITextEditorActionConstants.SELECT_ALL, action);
 
@@ -586,7 +545,7 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 			// create context menu
 			MenuManager manager= new MenuManager(null, null);
 			manager.setRemoveAllWhenShown(true);
-			manager.addMenuListener(mgr -> fillContextMenu(mgr));
+			manager.addMenuListener(this::fillContextMenu);
 
 			StyledText text= fPatternEditor.getTextWidget();
 			Menu menu= manager.createContextMenu(text);
@@ -636,7 +595,7 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 		private void updateButtons() {
 			StatusInfo status;
 
-			boolean valid= fNameText == null || fNameText.getText().trim().length() != 0;
+			boolean valid= fNameText == null || !fNameText.getText().trim().isEmpty();
 			if (!valid) {
 				status = new StatusInfo();
 				if (!fSuppressError)
@@ -705,7 +664,9 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 		@Override
 		protected IDialogSettings getDialogBoundsSettings() {
 			String sectionName= getClass().getName() + "_dialogBounds"; //$NON-NLS-1$
-			IDialogSettings settings= TextEditorPlugin.getDefault().getDialogSettings();
+			IDialogSettings settings = PlatformUI
+					.getDialogSettingsProvider(FrameworkUtil.getBundle(TemplatePreferencePage.class))
+					.getDialogSettings();
 			IDialogSettings section= settings.getSection(sectionName);
 			if (section == null)
 				section= settings.addNewSection(sectionName);
@@ -773,6 +734,9 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 	/** The context type registry. */
 	private ContextTypeRegistry fContextTypeRegistry;
 
+	private TextViewerAction fPatternViewerCopyAction;
+
+	private TextViewerAction fPatternViewerSelectAllAction;
 
 	/**
 	 * Creates a new template preference page.
@@ -1027,9 +991,10 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 	private TemplatePersistenceData[] getEnabledTemplates() {
 		List<TemplatePersistenceData> enabled= new ArrayList<>();
 		TemplatePersistenceData[] datas= fTemplateStore.getTemplateData(false);
-		for (int i= 0; i < datas.length; i++) {
-			if (datas[i].isEnabled())
-				enabled.add(datas[i]);
+		for (TemplatePersistenceData data : datas) {
+			if (data.isEnabled()) {
+				enabled.add(data);
+			}
 		}
 		return enabled.toArray(new TemplatePersistenceData[enabled.size()]);
 	}
@@ -1056,9 +1021,46 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 		data.heightHint= convertHeightInCharsToPixels(5);
 		control.setLayoutData(data);
 
+		addActionsToPatternViewer(viewer);
+
 		return viewer;
 	}
 
+	private void addActionsToPatternViewer(SourceViewer viewer) {
+		// create actions
+		fPatternViewerCopyAction = new TextViewerAction(viewer, ITextOperationTarget.COPY);
+		fPatternViewerCopyAction.setActionDefinitionId(ActionFactory.COPY.getCommandId());
+		fPatternViewerCopyAction.setText(TemplatesMessages.EditTemplateDialog_copy);
+
+		fPatternViewerSelectAllAction = new TextViewerAction(viewer, ITextOperationTarget.SELECT_ALL);
+		fPatternViewerSelectAllAction.setActionDefinitionId(ActionFactory.SELECT_ALL.getCommandId());
+		fPatternViewerSelectAllAction.setText(TemplatesMessages.EditTemplateDialog_select_all);
+
+		viewer.addSelectionChangedListener(this::updateCopyAction);
+		// create context menu
+		MenuManager manager = new MenuManager(null, null);
+		manager.setRemoveAllWhenShown(true);
+		manager.addMenuListener(this::fillPatternViewerContextMenu);
+
+		StyledText text = viewer.getTextWidget();
+		Menu menu = manager.createContextMenu(text);
+		text.setMenu(menu);
+	}
+
+	private void fillPatternViewerContextMenu(IMenuManager menu) {
+		menu.add(new Separator(ITextEditorActionConstants.GROUP_EDIT));
+		menu.appendToGroup(ITextEditorActionConstants.GROUP_EDIT, fPatternViewerCopyAction);
+		menu.appendToGroup(ITextEditorActionConstants.GROUP_EDIT, fPatternViewerSelectAllAction);
+	}
+
+	/**
+	 * @param event The event
+	 */
+	private void updateCopyAction(SelectionChangedEvent event) {
+		if (fPatternViewerCopyAction != null)
+			fPatternViewerCopyAction.update();
+	}
+
 	/**
 	 * Creates, configures and returns a source viewer to present the template
 	 * pattern on the preference page. Clients may override to provide a custom
@@ -1202,9 +1204,7 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 		if (newTemplate != null) {
 
 			if (!newTemplate.getName().equals(oldTemplate.getName()) &&
-				MessageDialog.openQuestion(getShell(),
-				TemplatesMessages.TemplatePreferencePage_question_create_new_title,
-				TemplatesMessages.TemplatePreferencePage_question_create_new_message))
+					openCreateNewOrRenameDialog())
 			{
 				data= new TemplatePersistenceData(newTemplate, true);
 				fTemplateStore.add(data);
@@ -1235,8 +1235,7 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 			if (file.exists()) {
 				try (InputStream input = new BufferedInputStream(new FileInputStream(file))) {
 					TemplatePersistenceData[] datas= reader.read(input, null);
-					for (int i= 0; i < datas.length; i++) {
-						TemplatePersistenceData data= datas[i];
+					for (TemplatePersistenceData data : datas) {
 						fTemplateStore.add(data);
 						String id= data.getId();
 						if (id == null) {
@@ -1436,6 +1435,15 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 		MessageDialog.openError(getShell(), title, message);
 	}
 
+	private boolean openCreateNewOrRenameDialog() {
+		MessageDialog dialog = new MessageDialog(getShell(),
+				TemplatesMessages.TemplatePreferencePage_question_create_new_title, null,
+				TemplatesMessages.TemplatePreferencePage_question_create_new_message, MessageDialog.QUESTION, 0,
+				TemplatesMessages.TemplatePreferencePage_question_create_new_button_create,
+				TemplatesMessages.TemplatePreferencePage_question_create_new_button_rename);
+		return dialog.open() == 0;
+	}
+
 	protected SourceViewer getViewer() {
 		return fPatternViewer;
 	}
